use crate::pgp::KnownKeys;
use crate::snippet::*;
use std::path::Path;
use std::sync::Mutex;
use tokio::prelude::*;
use tower_web::ServiceBuilder;
use tower_web::{derive_resource_impl, impl_web};

#[derive(Debug)]
struct Brownpaper {
    storage: SnippetStorage,
    known_keys: Mutex<KnownKeys>,
}

impl_web! {
    impl Brownpaper {
        #[get("/")]
        async fn root(&self) -> &'static str {
            "Post /new, Get /<id>"
        }

        #[post("/new")]
        async fn new(&self) -> Result<> {}

        #[get("/:id")]
        async fn snipptet(&self, id: String) -> Result<> {}

        #[get("/file/:file_id")]
        async fn snipptet_file(&self, file_id: String) -> Result<> {}
    }
}

#[cfg(not(debug_assertions))]
const STORAGE_DIR: &str = "/snips";

#[cfg(debug_assertions)]
const STORAGE_DIR: &str = "/tmp";

pub fn run() {
    let addr = "0.0.0.0:3000".parse().expect("Invalid address");
    println!("Listening on http://{}", addr);

    ServiceBuilder::new()
        .resource(Brownpaper {
            storage: SnippetStorage::new(Path::new(STORAGE_DIR)),
            known_keys: Mutex::new(
                KnownKeys::load_dir([STORAGE_DIR, "keys"].join("/"))
                    .expect("Failed to load PGP Keys"),
            ),
        })
        .run(&addr)
        .unwrap();
}
